/*
 * Copyright (C) 2015 Open Source Robotics Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
*/
#ifndef _GAZEBO_RENDERING_WINDOWMANAGER_PRIVATE_HH_
#define _GAZEBO_RENDERING_WINDOWMANAGER_PRIVATE_HH_

#include <vector>

namespace Ogre
{
  class RenderWindow;
}

namespace gazebo
{
  namespace rendering
  {
    /// \internal
    /// \brief Private data for the WindowManager class
    class WindowManagerPrivate
    {
      /// \brief All the render windows.
      public: std::vector<Ogre::RenderWindow *> windows;

      /// \brief Used to create unique names for the windows.
      public: static uint32_t windowCounter;
    };
  }
}
#endif
